"""Interface to TBProfiler report."""
from typing import TextIO

import json

import click

drug_list = [
    "isoniazid",
    "rifampicin",
    "ethambutol",
    "pyrazinamide",
    "streptomycin",
    "ethionamide",
    "fluoroquinolones",  # this is present in older versions of tbdb
    "amikacin",
    "capreomycin",
    "kanamycin",
    "para-aminosalicylic_acid",
    "cycloserine",
    "delamanid",
    "linezolid",
    "clofazimine",
    "bedaquiline",
    "aminoglycosides"  # this is present in newer versions of tbdb
    "levofloxacin",
    "moxifloxacin",
    "pretomanid"
]

drug_names = {
    "isoniazid": "Isoniazid",
    "rifampicin": "Rifampicin",
    "ethambutol": "Ethambutol",
    "pyrazinamide": "Pyrazinamide",
    "streptomycin": "Streptomycin",
    "ethionamide": "Ethionamide",
    "fluoroquinolones": "Fluoroquinolones",  # this is present in older versions of tbdb
    "amikacin": "Amikacin",
    "capreomycin": "Capreomycin",
    "kanamycin": "Kanamycin",
    "para-aminosalicylic_acid": "Para-aminosalicylic acid",
    "linezolid": "Linezolid",
    "cycloserine": "Cycloserine",
    "delamanid": "Delamanid",
    "clofazimine": "Clofazimine",
    "bedaquiline": "Bedaquiline",
    "aminoglycosides": "Aminoglycosides",  # this is present in newer versions of tbdb
    "levofloxacin": "Levofloxacin",
    "moxifloxacin": "Moxifloxacin",
    "pretomanid": "Pretomanid"
}


class TBProfilerReport:
    """Process tbprofiler_report."""

    def __init__(self, json_file: TextIO, _variants: list[list]):
        """initializer."""
        self.json_file = json_file
        self._variants = _variants

    def get_data(self) -> tuple[list[str], dict, int, str, dict[str, str]]:
        """Get TBProfiler data."""
        drug_resistance_list, call_positions = [], set()
        rrs_start, rrs_end = 1471846, 1473382
        rrs_variant_count = 0
        for variant_record in self._variants:
            POS = int(variant_record[17])
            if POS >= rrs_start and POS <= rrs_end:
                rrs_variant_count += 1
            call_positions.add(POS)
        tbprofiler_data = json.load(self.json_file)
        if "tbprofiler_version" in tbprofiler_data:
            tbprofiler_version = tbprofiler_data["tbprofiler_version"]
        else:
            tbprofiler_version = tbprofiler_data["pipeline"]["software_version"]
        tbprofiler_major_version= int(tbprofiler_version.split(".")[0])

        if tbprofiler_major_version < 3:
            tbdb_version = {}
            for key in ("name", "commit", "author", "date"):
                tbdb_version[key] = "Unknown"
        elif tbprofiler_major_version < 4:
            tbdb_version = tbprofiler_data["db_version"]
        else:
            tbdb_version = tbprofiler_data["pipeline"]["db_version"]  # dict with keys: name, commit, author, date
        dr_data = tbprofiler_data["dr_variants"]

        if tbprofiler_major_version < 4:
            main_lineage_key = "main_lin"
            sublineage_key = "sublin"
            lineage_key = "lin"
            fraction_key = "frac"
        else:
            main_lineage_key = "main_lineage"
            sublineage_key = "sub_lineage"
            lineage_key = "lineage"
            fraction_key = "fraction"
        lineage_details = {}
        for lineage in tbprofiler_data["lineage"]:
            lineage_details[lineage[lineage_key]] = (lineage[lineage_key],
                                                   lineage[fraction_key],
                                                   lineage["family"],
                                                   "" if lineage["rd"] is None else lineage["rd"])
        lineage_info = {
            "lineage": lineage_details[tbprofiler_data[main_lineage_key]]
        }
        if sublineage_key in tbprofiler_data:
            lineage_info["sub_lineage"] = lineage_details[tbprofiler_data[sublineage_key]]

        _drug_resistance = self._drug_resistance(
            dr_data, call_positions, tbprofiler_major_version
        )
        for drug_name in drug_names:
            if drug_name in _drug_resistance:
                drug_resistance_list.append(_drug_resistance[drug_name])
            else:
                drug_resistance_list.append(
                    {
                        "drug_name": drug_name,
                        "drug_human_name": drug_names[drug_name],
                        "resistant": False,
                    }
                )
        return (drug_resistance_list, lineage_info, rrs_variant_count, tbprofiler_version, tbdb_version)

    @staticmethod
    def _drug_resistance(_dr_data: dict, call_positions: list[int], tbprofiler_version: int) -> dict:
        dr_calls_seen, drug_resistance = set(), {}
        position_key = "genome_pos" if tbprofiler_version < 4 else "pos"
        gene_name_key = "gene" if tbprofiler_version < 4 else "gene_name"
        for record in _dr_data:
            if record[position_key] in dr_calls_seen:
                continue
            dr_calls_seen.add(record[position_key])
            if tbprofiler_version < 3:
                variant_drug_names = [record["drug"]]
            else:
                variant_drug_names = [drug["drug"] for drug in record["drugs"]]
            for drug_name in variant_drug_names:
                if drug_name not in drug_list:
                    click.secho(f"Unknown drug: {drug_name}\n", fg="yellow")
                    continue
                dr_record = drug_resistance.get(
                    drug_name,
                    {
                        "drug": drug_name,
                        "drug_human_name": drug_names[drug_name],
                        "resistant": True,
                        "snippy_agreement": True,
                        "variants": [],
                    },
                )
                if record[position_key] not in call_positions:
                    dr_record["snippy_agreement"] = False
                dr_record["variants"].append(
                    (record[gene_name_key], record["change"], round(record["freq"], 2))  # round to 2 digits
                )
                drug_resistance[drug_name] = dr_record
        return drug_resistance
